// povrender.cc
//
// cre: 2012-02-15
// aut: JGS
//
// (C) 2012 EVOLViA

#include "povrender.h"
#include "universe.h"
#include "physicalobject.h"
#include <fstream>
#include <sstream>
#include <cstdlib>


char const * const POVRender::s_colors[4] = {"Red", "Blue", "Yellow", "Pink"};
size_t const POVRender::s_numColors = sizeof(s_colors) / sizeof(*s_colors);


POVRender::POVRender(std::string const & basename, int width, int height)
:
	d_scenefile(basename + ".pov"),
	d_imagefile(basename + ".png"),
	d_width(width),
	d_height(height)
{}

void POVRender::operator() (Universe const & universe, double t)
{
	writeScene(universe, t);
	renderScene();
}

void POVRender::writeScene(Universe const &universe, double t)
{
	std::ofstream out(d_scenefile.c_str());

	out << "#version 3.6;\n"
		<< "#include \"colors.inc\"\n"
		<< "#include \"textures.inc\"\n\n";
	
	out << "camera {\n"
		<< "\tright x*image_width/image_height\n"
		<< "\tlook_at <0, 0, " << t << ">\n"
		<< "\tlocation <" << (-2 - 2 * t) << ", 1.8, " << (-3.5 + 7 * t) << ">\n"
		<< "}\n\n";

	out << "light_source {\n"
		<< "\t<1, 10, -4>\n"
		<< "\tcolor White\n"
		<< "}\n\n";

	out << "background {\n"
		<< "\tcolor Black\n"
		<< "}\n\n";

	out << "fog {\n"
		<< "\tdistance 20\n"
		<< "\tcolor rgb <0.5, 0.5, 0.75>\n"
		<< "}\n\n";

	out << "plane {\n"
		<< "\ty, 0\n"
		<< "\tpigment { checker Green, White }\n"
		<< "\tfinish { Metal }\n"
		<< "}\n\n";

	Universe::const_iterator it = universe.begin();
	int colorIdx = 0;
	while (it != universe.end())
	{
		out << "sphere {\n"
			<< "\t<" << (*it)->position().x << ", " << (*it)->position().y << ", " << (*it)->position().z << ">, 0.1\n"
			<< "\tpigment {\n"
			<< "\t\tcolor " << s_colors[colorIdx] << "\n"
			<< "\t\tfilter 0.8\n"
			<< "\t}\n"
			<< "\tfinish { Glass_Finish }\n"
			<< "\tinterior { Glass_Interior }\n"
			<< "}\n\n";
		it++;
		colorIdx++;
		colorIdx %= s_numColors;
	}
}

int POVRender::renderScene()
{
	std::ostringstream command;
	command << "povray"
		<< " +I" << d_scenefile
		<< " +O" << d_imagefile
		<< " +W" << d_width
		<< " +H" << d_height
		<< " +A";

	return system(command.str().c_str());
}


