// universe.cc
//
// cre: 2012-02-16
// aut: JGS
//
// (C) 2012 EVOLViA

#include "universe.h"
#include "physicalobject.h"
#include "vector3.h"

Universe::~Universe()
{
	std::list<PhysicalObject *>::iterator it = d_objects.begin();
	while (it != d_objects.end())
		delete *it++;
}

void Universe::step(double dt)
{
	// Apply forces
	std::list<PhysicalObject *>::iterator itOuter = d_objects.begin();
	while (itOuter != d_objects.end())
	{
		std::list<PhysicalObject *>::iterator itInner = itOuter;
		itInner++;
		while (itInner != d_objects.end())
		{
			double d = norm((*itInner)->position() - (*itOuter)->position());
			Vector3 r = ((*itInner)->position() - (*itOuter)->position()) / d;
			Vector3 F = -(d_G * (*itOuter)->mass() * (*itInner)->mass() / (d * d)) * r;

			(*itInner)->applyForce(F);
			(*itOuter)->applyForce(-F);

			itInner++;
		}
		itOuter++;
	}

	// Update
	itOuter = d_objects.begin();
	while (itOuter != d_objects.end())
		(*itOuter++)->step(dt);
}
