// universe.h
//
// cre: 2012-02-15
// aut: JGS
//
// (C) 2012 EVOLViA

#include <list>

class PhysicalObject;

class Universe
{
	std::list<PhysicalObject *>
		d_objects;

	// Gravitational constant
	double const
		d_G;

	public:

		~Universe();

		Universe(double G = 6.67384e-11)
		: d_G(G)
		{}

		// Add a new physical object to the universe. The pointer is
		// owned by the Universe.
		Universe & operator += (PhysicalObject * object)
		{
			d_objects.push_back(object); return *this;
		}

		void step(double dt);

		// Begin and end iterators to walk through the list of objects
		typedef std::list<PhysicalObject *>::const_iterator const_iterator;

		const_iterator begin() const
		{
			return d_objects.begin();
		}
		const_iterator end() const
		{
			return d_objects.end();
		}

		// returns the size of the universe, as total number of physical objects
		size_t size() const
		{
			return d_objects.size();
		}
};

