// vector3.h
//
// 3-dimensional vector class, "double" as base datatype
//
// cre: 2012-02-16
// aut: JGS
//
// (C) 2012 EVOLViA

#pragma once

#include <cmath>

struct Vector3
{
	double x, y, z;
	
	Vector3() 
	{} 

	Vector3(double cx, double cy, double cz) 
	: x(cx), y(cy), z(cz) 
	{}

	Vector3 & operator += (double c)
	{
		x += c; y +=c; z += c; return *this;
	}

	Vector3 & operator += (Vector3 const & v)
	{
		x += v.x; y += v.y; z += v.z; return *this;
	}

	Vector3 & operator -= (double c)
	{
		x -= c; y -=c; z -= c; return *this;
	}

	Vector3 & operator -= (Vector3 const & v)
	{
		x -= v.x; y -= v.y; z -= v.z; return *this;
	}

	Vector3 operator - () const
	{
		return Vector3(-x, -y, -z);
	}

	Vector3 operator * (double c) const
	{
		return Vector3(c * x, c * y, c * z);
	}

	Vector3 operator / (double c) const
	{
		return Vector3(x / c, y / c, z / c);
	}
};

inline Vector3 operator + (Vector3 const & a, Vector3 const & b)
{
	return Vector3(a.x + b.x, a.y + b.y, a.z + b.z);
}

inline Vector3 operator - (Vector3 const & a, Vector3 const & b)
{
	return Vector3(a.x - b.x, a.y - b.y, a.z - b.z);
}

inline Vector3 operator * (double c, Vector3 const & v)
{
	return (v * c);
}

inline Vector3 operator / (double c, Vector3 const & v)
{
	return (v / c);
}

inline double dot(Vector3 const & a, Vector3 const & b)
{
	return (a.x * b.x + a.y * b.y + a.z * b.z);
}

inline Vector3 cross(Vector3 const & a, Vector3 const & b)
{
	return Vector3(a.y * b.z - a.z * b.y, a.z * b.x - a.x * b.z, a.x * b.y - a.y * b.x);
}

inline double norm(Vector3 const & v)
{
	return sqrt(dot(v, v));
}

